@@ -13,7 +13,7 @@
 """Property sheets
 """
 
-import six
+import html
 
 from AccessControl.class_init import InitializeClass
 from AccessControl.Permissions import access_contents_information
@@ -33,13 +33,7 @@ from zExceptions import BadRequest
 from ZPublisher.Converters import type_converters
 
 
-try:
-    from html import escape
-except ImportError:  # PY2
-    from cgi import escape
-
-
-class Virtual(object):
+class Virtual:
     """A virtual propertysheet stores it's properties in it's instance."""
 
     def __init__(self):
@@ -138,7 +132,7 @@ class PropertySheet(Traversable, Persistent, Implicit, DAVPropertySheetMixin):
 
         if id in self.__reserved_ids:
             raise ValueError(
-                "'%s' is a reserved Id (forbidden Ids are: %s) " % (
+                "'{}' is a reserved Id (forbidden Ids are: {}) ".format(
                     id, self.__reserved_ids))
 
         self.id = id
@@ -161,7 +155,7 @@ class PropertySheet(Traversable, Persistent, Implicit, DAVPropertySheetMixin):
 
     def valid_property_id(self, id):
         if not id or id[:1] == '_' or (id[:3] == 'aq_') \
-           or (' ' in id) or escape(id, True) != id:
+           or (' ' in id) or html.escape(id, True) != id:
             return 0
         return 1
 
@@ -203,7 +197,10 @@ class PropertySheet(Traversable, Persistent, Implicit, DAVPropertySheetMixin):
         # systems.
         self._wrapperCheck(value)
         if not self.valid_property_id(id):
-            raise BadRequest('Invalid property id, %s.' % escape(id, True))
+            raise BadRequest(
+                'Invalid property id, %s.' %
+                html.escape(
+                    id, True))
 
         if not self.property_extensible_schema__():
             raise BadRequest(
@@ -214,7 +211,7 @@ class PropertySheet(Traversable, Persistent, Implicit, DAVPropertySheetMixin):
             if not (id == 'title' and id not in self.__dict__):
                 raise BadRequest(
                     'Invalid property id, <em>%s</em>. It is in use.' %
-                    escape(id, True))
+                    html.escape(id, True))
         if meta is None:
             meta = {}
         prop = {'id': id, 'type': type, 'meta': meta}
@@ -242,11 +239,11 @@ class PropertySheet(Traversable, Persistent, Implicit, DAVPropertySheetMixin):
         self._wrapperCheck(value)
         if not self.hasProperty(id):
             raise BadRequest('The property %s does not exist.' %
-                             escape(id, True))
+                             html.escape(id, True))
         propinfo = self.propertyInfo(id)
         if 'w' not in propinfo.get('mode', 'wd'):
-            raise BadRequest('%s cannot be changed.' % escape(id, True))
-        if isinstance(value, (six.string_types, six.binary_type)):
+            raise BadRequest('%s cannot be changed.' % html.escape(id, True))
+        if isinstance(value, (str, bytes)):
             proptype = propinfo.get('type', 'string')
             if proptype in type_converters:
                 value = type_converters[proptype](value)
@@ -259,7 +256,7 @@ class PropertySheet(Traversable, Persistent, Implicit, DAVPropertySheetMixin):
                 props.append(prop)
             pself._properties = tuple(props)
 
-        if isinstance(value, list):
+        if type(value) == list:
             value = tuple(value)
         setattr(self.v_self(), id, value)
 
@@ -268,14 +265,14 @@ class PropertySheet(Traversable, Persistent, Implicit, DAVPropertySheetMixin):
         # given id does not exist, a ValueError is raised.
         if not self.hasProperty(id):
             raise BadRequest('The property %s does not exist.' %
-                             escape(id, True))
+                             html.escape(id, True))
         vself = self.v_self()
         if hasattr(vself, '_reserved_names'):
             nd = vself._reserved_names
         else:
             nd = ()
         if ('d' not in self.propertyInfo(id).get('mode', 'wd')) or (id in nd):
-            raise BadRequest('%s cannot be deleted.' % escape(id, True))
+            raise BadRequest('%s cannot be deleted.' % html.escape(id, True))
         delattr(vself, id)
         pself = self.p_self()
         pself._properties = tuple(
@@ -303,7 +300,10 @@ class PropertySheet(Traversable, Persistent, Implicit, DAVPropertySheetMixin):
         for p in self._propertyMap():
             if p['id'] == id:
                 return p
-        raise ValueError('The property %s does not exist.' % escape(id, True))
+        raise ValueError(
+            'The property %s does not exist.' %
+            html.escape(
+                id, True))
 
     def _propertyMap(self):
         """Return a tuple of mappings, giving meta-data for properties."""
@@ -367,7 +367,7 @@ class PropertySheet(Traversable, Persistent, Implicit, DAVPropertySheetMixin):
             if self.hasProperty(name):
                 if 'w' not in propdict[name].get('mode', 'wd'):
                     raise BadRequest('%s cannot be changed' %
-                                     escape(name, True))
+                                     html.escape(name, True))
                 self._updateProperty(name, value)
         message = 'Your changes have been saved.'
         return self.manage(self, REQUEST, manage_tabs_message=message)
