@@ -16,7 +16,7 @@
 import unittest
 
 
-class UnitTestSecurityPolicy(object):
+class UnitTestSecurityPolicy:
     """
         Stub out the existing security policy for unit testing purposes.
     """
@@ -29,7 +29,7 @@ class UnitTestSecurityPolicy(object):
         return 1
 
 
-class CruelSecurityPolicy(object):
+class CruelSecurityPolicy:
     """Denies everything
     """
     #   Standard SecurityPolicy interface
@@ -41,7 +41,7 @@ class CruelSecurityPolicy(object):
         return 0
 
 
-class ProtectedMethodSecurityPolicy(object):
+class ProtectedMethodSecurityPolicy:
     """Check security strictly on bound methods.
     """
     def validate(self, accessed, container, name, value, *args):
@@ -427,8 +427,26 @@ class TestTraverse(unittest.TestCase):
         self.assertTrue(
             self.folder1.unrestrictedTraverse('+something') == 'plus')
 
+    def testTraverseWrongType(self):
+        with self.assertRaises(TypeError):
+            self.folder1.unrestrictedTraverse(1)
+        with self.assertRaises(TypeError):
+            self.folder1.unrestrictedTraverse(b"foo")
+        with self.assertRaises(TypeError):
+            self.folder1.unrestrictedTraverse(["foo", b"bar"])
+        with self.assertRaises(TypeError):
+            self.folder1.unrestrictedTraverse(("foo", None))
+        with self.assertRaises(TypeError):
+            self.folder1.unrestrictedTraverse({1, "foo"})
 
-class SimpleClass(object):
+    def testTraverseEmptyPath(self):
+        self.assertEqual(self.folder1.unrestrictedTraverse(None), self.folder1)
+        self.assertEqual(self.folder1.unrestrictedTraverse(""), self.folder1)
+        self.assertEqual(self.folder1.unrestrictedTraverse([]), self.folder1)
+        self.assertEqual(self.folder1.unrestrictedTraverse({}), self.folder1)
+
+
+class SimpleClass:
     """Class with no __bobo_traverse__."""
 
 
