@@ -24,9 +24,6 @@ from doctest import REPORT_NDIFF
 from functools import partial
 from io import BytesIO
 
-from six import PY2
-from six import text_type
-
 import transaction
 from Testing.ZopeTestCase import Functional
 from Testing.ZopeTestCase import FunctionalTestCase
@@ -43,11 +40,7 @@ from ZPublisher.httpexceptions import HTTPExceptionHandler
 from ZPublisher.utils import basic_auth_encode
 
 
-if sys.version_info >= (3, ):
-    basestring = str
-
-
-class HTTPHeaderOutput(object):
+class HTTPHeaderOutput:
 
     status = '200'
     reason = 'OK'
@@ -62,14 +55,14 @@ class HTTPHeaderOutput(object):
         self.status, self.reason = status, reason
 
     def setResponseHeaders(self, mapping):
-        self.headers.update(dict(
-            [('-'.join([s.capitalize() for s in name.split('-')]), v)
-             for name, v in mapping.items()
-             if name.lower() not in self.omit]
-        ))
+        self.headers.update({
+            '-'.join([s.capitalize() for s in name.split('-')]): v
+            for name, v in mapping.items()
+            if name.lower() not in self.omit
+        })
 
     def appendResponseHeaders(self, lst):
-        if lst and isinstance(lst[0], basestring):
+        if lst and isinstance(lst[0], str):
             headers = [split_header(header) for header in lst]
         else:
             headers = lst
@@ -83,7 +76,7 @@ class HTTPHeaderOutput(object):
         out = ["%s: %s" % header for header in self.headers.items()]
         out.extend(["%s: %s" % header for header in self.headersl])
         out.sort()
-        out.insert(0, "%s %s %s" % (self.protocol, self.status, self.reason))
+        out.insert(0, f"{self.protocol} {self.status} {self.reason}")
         return '\n'.join(out)
 
 
@@ -100,7 +93,7 @@ class DocResponseWrapper(ResponseWrapper):
     def __str__(self):
         body = self._decode(self.getBody())
         if body:
-            return "%s\n\n%s" % (self.header_output, body)
+            return f"{self.header_output}\n\n{body}"
         return "%s\n" % (self.header_output)
 
 
@@ -138,7 +131,7 @@ def http(request_string, handle_errors=True):
 
     This is used for HTTP doc tests.
     """
-    from six.moves.urllib.parse import unquote_to_bytes
+    from urllib.parse import unquote
 
     from ZPublisher.HTTPRequest import WSGIRequest as Request
     from ZPublisher.HTTPResponse import WSGIResponse
@@ -155,6 +148,7 @@ def http(request_string, handle_errors=True):
     command_line = request_string[:newline].rstrip()
     request_string = request_string[newline + 1:]
     method, path, protocol = command_line.split()
+    path = unquote(path)
 
     env = {
         'HTTP_HOST': 'localhost',
@@ -163,14 +157,13 @@ def http(request_string, handle_errors=True):
         'SERVER_PROTOCOL': protocol,
     }
 
-    query = ''
-    if '?' in path:
-        path, query = path.split("?", 1)
-    if PY2:
-        env['PATH_INFO'] = unquote_to_bytes(path)
+    p = path.split('?', 1)
+    if len(p) == 1:
+        env['PATH_INFO'] = p[0]
+    elif len(p) == 2:
+        [env['PATH_INFO'], env['QUERY_STRING']] = p
     else:
-        env['PATH_INFO'] = unquote_to_bytes(path).decode('latin-1')
-    env['QUERY_STRING'] = query
+        raise TypeError('')
 
     header_output = HTTPHeaderOutput(
         protocol, ('x-content-type-warning', 'x-powered-by'))
@@ -182,7 +175,7 @@ def http(request_string, handle_errors=True):
     headers = msg.items()
     body = msg.get_payload()
 
-    if isinstance(body, text_type):
+    if isinstance(body, str):
         body = body.encode('utf-8')
 
     # Store request body without headers
@@ -203,6 +196,7 @@ def http(request_string, handle_errors=True):
 
     outstream = BytesIO()
     response = WSGIResponse(stdout=outstream, stderr=sys.stderr)
+    request = Request(instream, env, response)
 
     env['wsgi.input'] = instream
     wsgi_headers = BytesIO()
@@ -215,10 +209,7 @@ def http(request_string, handle_errors=True):
         wsgi_headers.write(headers)
         wsgi_headers.write(b'\r\n\r\n')
 
-    publish = partial(
-        publish_module,
-        _request_factory=Request,
-        _response=response)
+    publish = partial(publish_module, _request=request, _response=response)
     if handle_errors:
         publish = HTTPExceptionHandler(publish)
 
@@ -235,7 +226,7 @@ def http(request_string, handle_errors=True):
         response, outstream, path, header_output, wsgi_result, wsgi_headers)
 
 
-class ZopeSuiteFactory(object):
+class ZopeSuiteFactory:
 
     def __init__(self, *args, **kw):
         self._args = args
