@@ -42,6 +42,10 @@ from ZPublisher.BaseResponse import BaseResponse
 from ZPublisher.Iterators import IStreamIterator
 from ZPublisher.Iterators import IUnboundStreamIterator
 
+from .cookie import convertCookieParameter
+from .cookie import getCookieParamPolicy
+from .cookie import getCookieValuePolicy
+
 
 # This may get overwritten during configuration
 default_encoding = 'utf-8'
@@ -311,9 +315,12 @@ class HTTPBaseResponse(BaseResponse):
         else:
             cookie = cookies[name] = {}
         for k, v in kw.items():
+            k, v = convertCookieParameter(k, v)
             cookie[k] = v
         cookie['value'] = value
-        cookie['quoted'] = quoted
+        # RFC6265 makes quoting obsolete
+        # cookie['quoted'] = quoted
+        getCookieParamPolicy().check_consistency(name, cookie)
 
     def appendCookie(self, name, value):
         """ Set an HTTP cookie.
@@ -498,11 +505,13 @@ class HTTPBaseResponse(BaseResponse):
         If the body is a 2-element tuple, then it will be treated
         as (title,body)
 
-        If body is unicode, encode it.
+        If body has an 'asHTML' method, replace it by the result of that
+        method.
 
-        If body is not a string or unicode, but has an 'asHTML' method, use
-        the result of that method as the body;  otherwise, use the 'str'
-        of body.
+        If body is now bytes, bytearray or memoryview, convert it to bytes.
+        Else, either try to convert it to bytes or use an intermediate string
+        representation which is then converted to bytes, depending on the
+        content type.
 
         If is_error is true, format the HTML as a Zope error message instead
         of a generic HTML page.
@@ -524,15 +533,17 @@ class HTTPBaseResponse(BaseResponse):
         if hasattr(body, 'asHTML'):
             body = body.asHTML()
 
-        if isinstance(body, str):
-            body = self._encode_unicode(body)
-        elif isinstance(body, bytes):
-            pass
-        else:
+        content_type = self.headers.get('content-type')
+
+        if isinstance(body, (bytes, bytearray, memoryview)):
+            body = bytes(body)
+        elif content_type is not None and not content_type.startswith('text/'):
             try:
                 body = bytes(body)
             except (TypeError, UnicodeError):
-                body = self._encode_unicode(str(body))
+                pass
+        if not isinstance(body, bytes):
+            body = self._encode_unicode(str(body))
 
         # At this point body is always binary
         b_len = len(body)
@@ -553,8 +564,6 @@ class HTTPBaseResponse(BaseResponse):
             else:
                 self.body = body
 
-        content_type = self.headers.get('content-type')
-
         if content_type is None:
             if self.isHTML(body):
                 content_type = f'text/html; charset={self.charset}'
@@ -669,45 +678,19 @@ class HTTPBaseResponse(BaseResponse):
 
     def _cookie_list(self):
         cookie_list = []
+        dump = getCookieValuePolicy().dump
+        parameters = getCookieParamPolicy().parameters
         for name, attrs in self.cookies.items():
-
-            # Note that as of May 98, IE4 ignores cookies with
-            # quoted cookie attr values, so only the value part
-            # of name=value pairs may be quoted.
-
-            v = quote(attrs['value'])
-            if attrs.get('quoted', True):
-                cookie = f'{name}="{v}"'
-            else:
-                cookie = f'{name}={v}'
-            for name, v in attrs.items():
-                name = name.lower()
-                if name == 'expires':
-                    cookie = f'{cookie}; Expires={v}'
-                elif name == 'domain':
-                    cookie = f'{cookie}; Domain={v}'
-                elif name == 'path':
-                    cookie = f'{cookie}; Path={v}'
-                elif name == 'max_age':
-                    cookie = f'{cookie}; Max-Age={v}'
-                elif name == 'comment':
-                    cookie = f'{cookie}; Comment={v}'
-                elif name == 'secure' and v:
-                    cookie = '%s; Secure' % cookie
-                # Some browsers recognize this cookie attribute
-                # and block read/write access via JavaScript
-                elif name == 'http_only' and v:
-                    cookie = '%s; HTTPOnly' % cookie
-                # Some browsers recognize the SameSite cookie attribute
-                # and do not send the cookie along with cross-site requests
-                # providing some protection against CSRF attacks
-                # https://tools.ietf.org/html/draft-west-first-party-cookies-07
-                elif name == 'same_site' and v:
-                    cookie = f'{cookie}; SameSite={v}'
+            params = []
+            for pn, pv in parameters(name, attrs):
+                if pv is None:
+                    continue
+                params.append(pn if pv is True else f"{pn}={pv}")
+            cookie = "{}={}".format(name, dump(name, attrs["value"]))
+            if params:
+                cookie += "; " + "; ".join(params)
+            # Should really check size of cookies here!
             cookie_list.append(('Set-Cookie', cookie))
-
-        # Should really check size of cookies here!
-
         return cookie_list
 
     def listHeaders(self):
@@ -719,7 +702,7 @@ class HTTPBaseResponse(BaseResponse):
         """
 
         result = [
-            ('X-Powered-By', 'Zope (www.zope.org), Python (www.python.org)')
+            ('X-Powered-By', 'Zope (www.zope.dev), Python (www.python.org)')
         ]
 
         encode = header_encoding_registry.encode
