@@ -1,4 +1,5 @@
 import os
+from typing import Optional, Tuple, List
 from configparser import RawConfigParser
 
 
@@ -20,21 +21,14 @@ def generate(in_, requirements_file, constraints_file):
 
     requirements = []
     constraints = []
-    versions = parser.items('versions')
     zope_requirement = (
         '-e git+https://github.com/zopefoundation/Zope.git@master#egg=Zope\n')
-    for name, pin in versions:
-        if name == 'Zope':
-            if pin:
-                zope_requirement = 'Zope==%s\n' % pin
-            continue
-
-        if not pin:
-            continue
-
-        spec = f'{name}=={pin}'
-        requirements.append(spec + '\n')
-        constraints.append(spec + '\n')
+    zope_requirement = _generate(
+        parser.items('versions'), None, requirements, constraints,
+        zope_requirement)
+    zope_requirement = _generate(
+        parser.items('versions:python36'), '3.6', requirements, constraints,
+        zope_requirement)
 
     with open(out_file_requirements, 'w') as fd:
         fd.write(zope_requirement)
@@ -45,6 +39,35 @@ def generate(in_, requirements_file, constraints_file):
             fcon.write(con)
 
 
+def _generate(
+    versions: List[Tuple[str, str]],
+    python_version: Optional[str],
+    requirements: List[str],
+    constraints: List[str],
+    zope_requirement: str
+) -> str:
+    """Generate requirements and constraints for a specific Python version.
+
+    If ``python_version`` is falsy, generate for all python versions.
+    Returns a probably changed ``zope_requirement``.
+    """
+    for name, pin in versions:
+        if name == 'Zope':
+            if pin:
+                zope_requirement = 'Zope==%s\n' % pin
+            continue
+
+        if not pin:
+            continue
+
+        spec = f'{name}=={pin}'
+        if python_version:
+            spec = f"{spec}; python_version == '{python_version}'"
+        requirements.append(spec + '\n')
+        constraints.append(spec + '\n')
+    return zope_requirement
+
+
 def main():
     generate('versions-prod.cfg', 'requirements-full.txt', 'constraints.txt')
 
